import Util from "../utils/Util";
import LabRegister from "./LabRegister";
export default {
    mixins: [LabRegister],
    inject: {
        findElement: {
            from: 'findElement',
            default: function (){
                return function () {
                    return []
                }
            }
        },
        openWindow: {
            from: 'openWindow',
            default: function (){
                return function () {
                    return false
                }
            }
        },
        closeWindow: {
            from: 'closeWindow',
            default: function (){
                return function () {
                    return false
                }
            }
        }
    },
    created() {
        this.dispatchEvent('created')
            .then(()=>{})
    },
    methods: {
        /**
         * 组件自身事件触发
         * @param trigger      目标事件
         * @param args         参数
         * @return {Promise<unknown>}
         */
        dispatchEvent(trigger, args) {
            return new Promise(resolve => {
                if (!this.element) {
                    resolve({code: 200})
                    return;
                }
                const { events } = this.element
                if (Util.isEmpty(events) || Util.isEmpty(trigger)){
                    resolve({code: 202, msg: '未匹配到组件事件'})
                } else {
                    const targets = events.filter(item => item.trigger === trigger && !Util.isEmpty(item.eventScript))
                    if (Util.isEmpty(targets)){
                        resolve({code: 203,msg: '未匹配到组件目标事件'})
                        return
                    }
                    const promises = []
                    targets.forEach(target => {
                        promises.push(this.executeJs(target.eventScript, args))
                    })
                    Promise.all(promises)
                        .then((ress) => {
                            for (let i = 0; i < ress.length; i++) {
                                const res = this.adapterRes(ress[i])
                                if (res.code !== 200) {
                                    resolve(res)
                                    return
                                }
                            }
                            resolve(this.adapterRes(ress[0]))
                        })
                }
            })
        },
        /**
         * js 执行
         * @param script        脚本
         * @param args          参数
         * @return {Promise<unknown>}
         */
        executeJs(script, args) {
            return new Promise((resolve) => {
                if (Util.isEmpty(script)) {
                    resolve({ code: 200 })
                } else {
                    const { element, modelData } = this
                    if (Util.isEmpty(args)) {
                        args = { element, modelData, $this: this, parentElement: this.parentElement, parentView: this.parentView }
                    } else {
                        if (!args.element) {
                            args.element = element
                        }
                        if (!args.modelData) {
                            args.modelData = modelData
                        }
                        if (!args.$this) {
                            args.$this = this
                        }
                        if (!args.parentElement) {
                            args.parentElement = this.parentElement
                        }
                        if (!args.parentView) {
                            args.parentView = this.parentView
                        }
                    }
                    // 方法回调
                    if (script instanceof Function) {
                        script(args)
                        resolve({ code: 200 })
                        return
                    }
                    // 异步执行
                    const execDefine = ['args']
                    const execArgs = [args]
                    execDefine.push(`return new Promise((resolve) => { \n ${script} })`)
                    const exFun = Function.apply(this, execDefine)
                    exFun.apply(this, execArgs)
                        .then(res => {
                            resolve(this.adapterRes(res))
                        })
                        .catch((e) => {
                            console.error(script, e)
                            resolve({ code: 500, msg: `JS脚本执行失败` })
                        })
                }
            })
        },
        /**
         * 触发组件事件
         * @param key           组件mapper
         * @param trigger       目标事件
         * @param matchObj      详细匹配对象
         * @return {Promise<unknown>}
         */
        dispatchElementEvent(key, trigger, matchObj) {
            return new Promise(resolve => {
                if (Util.isEmpty(key) || Util.isEmpty(trigger)){
                    resolve({code: 200})
                } else {
                    const $matchElement= this.findElement(key, matchObj)
                    if (Util.isEmpty($matchElement)){
                        resolve({code: 201, msg: '未发现组件'})
                    } else {
                        $matchElement.dispatchEvent(trigger)
                            .then((res) => {
                                resolve(this.adapterRes(res))
                        })
                    }
                }
            })
        },
        /**
         * 结果适配。确保方法出口都是{ code: xxx, msg: xxx, data: xxx } 结构`
         * @param res
         */
        adapterRes(res) {
            if (this.$Util.isObj(res)) {
                const keys = Object.keys(res)
                if (keys.includes('code') && keys.includes('msg')) {
                    return res
                } else if (keys.includes('msg')) {
                    Object.assign(res, { code: 500 })
                } else if (keys.includes('code')) {
                    if (res.code !== 200) {
                        Object.assign(res, { msg: '操作失败' })
                    } else {
                        return res
                    }
                }
            }
            return { code: 200, data: res }
        }
    }
}
