import axios from 'axios';
import AuthStorage from "./AuthStorage";
// 创建 axios 实例
const service = axios.create({
	baseURL: process.env.VUE_APP_BASE_API,
	timeout: 1000 * 60 * 10,
	headers: { 'Content-Type': 'application/json' },
});

// 添加请求拦截器
service.interceptors.request.use(
	(config) => {
		// 在发送请求之前做些什么 token
		const accessToken = AuthStorage.getAccessToken();
		if (accessToken && config.url) {
			if (!config.url.includes('http://','https://')) {
				config.headers.common['Authorization'] = `Bearer ${accessToken}`;
			}
		}
		console.debug("请求:" + config.url)
		return config;
	},
	(error) => {
		// 对请求错误做些什么
		return Promise.reject(error);
	}
);

// 添加响应拦截器
service.interceptors.response.use(
	(response) => {
		// 对响应数据做点什么
		const res = response.data;
		if (res.code && res.code !== 0) {
			return Promise.resolve(res);
		} else {
			return response.data;
		}
	},
	(error) => {
		if (error.response) {
			const response = error.response
			console.debug("错误响应:" + response)
			if (response.data && response.data.code) {
				return response.data;
			} else if (response.status) {
				return Promise.reject({ code: response.status, msg: response.data.error});
			} else {
				return Promise.reject(error);
			}
		} else if (error.message) {
			console.debug("错误响应:" + error.message)
			return Promise.reject({ code: 500, msg: error.message});
		} else {
			console.debug("错误响应:" + error)
			return Promise.reject({ code: 500, msg: '请求失败'});
		}
	}
);

// 导出 axios 实例
export default service;
